@@ -4,7 +4,7 @@ use warnings;
 use strict;
 
 use vars qw( $VERSION );
-$VERSION = '0.16';
+$VERSION = '0.17';
 
 use Carp 'croak';
 use HTML::PrettyPrinter;
@@ -13,75 +13,90 @@ use HTML::TreeBuilder;
 #
 # Changes
 #
-# Vers 0.16 - 5/25/04 10:16:33 PST
-# o Added benchmarking (using Time::HiRes if available)
-# o More Unicode support
-# o Lots has been moved to HTML::WikiConverter::Dialect module
+# version: 0.17
+# date:    Wed 7/7/04 12:24:11 PST
+# changes:
+# - More documentation
+# - Update test suite
+# - Remove warnings reported by cpan testers
 #
-# Vers 0.15 - 5/20/04
-# o Added support for wiki dialects via HTML::WikiConverter::Dialect interface
-# o Added HTML::WikiConverter::Dialect
-# o Added HTML::WikiConverter::Dialect::MediaWiki
-# o Added HTML::WikiConverter::Dialect::PhpWiki
-# o Added HTML::WikiConverter::Dialect::Kwiki
-# o Fixed spacing issues in tidy_whitespace
-# o Added handling of container, block, and line elements
-# o Now supports multiply-indented blocks
+# version: 0.16
+# date:    Fri 5/15/04 10:16:33 PST
+# changes:
+# - Added benchmarking (using Time::HiRes if available)
+# - More Unicode support
+# - Lots has been moved to HTML::WikiConverter::Dialect module
 #
-# Vers 0.14 - 5/17/04
-# o Changed 'wikify_default' to 'passthru' for semantic clarity
-# o NOWIKI blocks are no longer preserved -- they shouldn't
+# version: 0.15
+# date:    Sun 5/20/04 14:32:33 PST
+# changes:
+# - Added support for wiki dialects via HTML::WikiConverter::Dialect interface
+# - Added HTML::WikiConverter::Dialect
+# - Added HTML::WikiConverter::Dialect::MediaWiki
+# - Added HTML::WikiConverter::Dialect::PhpWiki
+# - Added HTML::WikiConverter::Dialect::Kwiki
+# - Fixed spacing issues in tidy_whitespace
+# - Added handling of container, block, and line elements
+# - Now supports multiply-indented blocks
+#
+# version: 0.14
+# date:    Thu 5/17/04 15:52:41 PST
+# changes:
+# - Changed 'wikify_default' to 'passthru' for semantic clarity
+# - NOWIKI blocks are no longer preserved -- they shouldn't
 #   appear in the HTML input, only in the WC output
-# o Bug fix: Add newline to HTML source before wikification --
+# - Bug fix: Add newline to HTML source before wikification --
 #   avoids apparent bugs in HTML::TreeBuilder that prevent proper
 #   tag nesting
-# o Added trim method to encapsulate whitespace trimming
-# o '_elem_has_ancestor' now accepts a compiled regexp or
+# - Added trim method to encapsulate whitespace trimming
+# - '_elem_has_ancestor' now accepts a compiled regexp or
 #   a tag name
-# o If a regexp with capturing parens is passed into
+# - If a regexp with capturing parens is passed into
 #   '_elem_has_ancestor', then captured values will be returned
 #   on a successful match
-# o Added support for nested lists (though mixed UL/OL lists are
+# - Added support for nested lists (though mixed UL/OL lists are
 #   not handled correctly)
-# o Bug fix: extra whitespace in PRE blocks is no longer trimmed
+# - Bug fix: extra whitespace in PRE blocks is no longer trimmed
 #   (required $parent parameter to 'wikify' method)
-# o Ensures that PRE blocks have at least one single space at 
+# - Ensures that PRE blocks have at least one single space at 
 #   the start of each line contained within. So "<PRE>test</PRE>"
 #   becomes " test" (on its own line).
-# o CENTER and SMALL tags are now preserved (passthru support)
-# o Added 'escape_wikitext' method to preserve "{{...}}" blocks
+# - CENTER and SMALL tags are now preserved (passthru support)
+# - Added 'escape_wikitext' method to preserve "{{...}}" blocks
 #   such as "{{msg:stub}}" or "{{NUMBEROFARTICLES}}"
-# o Bug fix: Add leading space before wiki links
-# o Can now produce "[[programming language]]s" wiki links
+# - Bug fix: Add leading space before wiki links
+# - Can now produce "[[programming language]]s" wiki links
 #
-# Vers 0.12 - 5/14/04
-# o Bug fix: removed reference to non-existent 'has_parent'
+# version: 0.12
+# date:    Mon 5/14/04 10:12:11 PST
+# changes:
+# - Bug fix: removed reference to non-existent 'has_parent'
 #   method within '_elem_has_ancestor'
-# o Bug fix: fixed potential bug in 'wikify_list_item'
+# - Bug fix: fixed potential bug in 'wikify_list_item'
 #   which used $node->parent->tag eq '...' instead of
 #   _elem_has_ancestor($node, '...')
-# o Added support for definition lists
-# o Added support for indentation
-# o Replace code handler for P tag with flank handler
-# o Replace code handler for OL/UL tags with flank handlers
-# o Renamed 'wikify_heading' to 'wikify_h' for consistency
+# - Added support for definition lists
+# - Added support for indentation
+# - Replace code handler for P tag with flank handler
+# - Replace code handler for OL/UL tags with flank handlers
+# - Renamed 'wikify_heading' to 'wikify_h' for consistency
 #   with other 'wikify_*' handlers
-# o NOWIKI blocks are now preserved
-# o Introduced beginnings of Unicode support with the
+# - NOWIKI blocks are now preserved
+# - Introduced beginnings of Unicode support with the
 #   use of HTML::Entities
 #
-# Vers 0.11 - 5/10/04
-# o Added wikify_default -- a default handler for
+# version: 0.11
+# date:    Thu 5/10/04 15:06:57 PST
+# changes:
+# - Added wikify_default -- a default handler for
 #   tags that should be preserved. Tags without
 #   handlers are removed from the wiki markup. So
 #   for example, "<TAG>content</TAG>" becomes 
 #   "content". By assigning the TAG to wikify_default,
 #   the resulting wiki markup is "<tag>content</tag>".
 #   All attributes are preserved as well.
-# o Added wikify_span
-# o New tags handled
-#   - FONT, SUP, SUB are now preserved in wiki markup (see wikify_default)
-#   - SPAN: attempts to convert into FONT (see wikify_span)
+# - Added wikify_span
+# - New tags handled (FONT, SUP, SUB, SPAN->FONT conversion)
 #
 
 =head1 NAME
@@ -101,9 +116,17 @@ HTML::WikiConverter - An HTML-to-wiki markup converter
 There are lots of programs out there that convert wiki markup into
 HTML, but relatively few that convert HTML into wiki
 markup. HTML::WikiConverter falls into the latter category, converting
-HTML source into wiki markup. The resulting markup is suitable for
-most wiki engines, but is admittedly targeted for MediaWiki (on which
-Wikipedia runs -- see http://wikipedia.org)
+HTML source into wiki markup.
+
+HTML to wiki conversion may be used in conjunction with in-browser
+WYSIWYG text editors to provide a WYSIWYG wiki interface. For example,
+a wiki edit page could allow users to modify text using a WYSIWYG
+editor such as HTMLArea (see
+http://dynarch.com/mishoo/htmlarea.epl). When the page is saved, HTML
+is submitted to the server, which can then use HTML::WikiConverter to
+translate the HTML back to its wikitext equivalent. Some alternative
+solutions are enumerated at
+http://meta.wikipedia.org/wiki/WYSIWYG_editor.
 
 =head1 METHODS
 
@@ -116,14 +139,23 @@ Wikipedia runs -- see http://wikipedia.org)
 Accepts a list of attribute name/value pairs and returns
 a new HTML::WikiConverter object. Allowed attribute names:
 
- file     - (scalar) name of HTML file to convert to wikitext
- html     - (scalar) HTML source to convert
- base_url - (scalar) base URL used to make absolute URLs
- dialect  - (scalar) wiki engine target, either MediaWiki
-                     or PhpWiki (default: MediaWiki)
+ file      - (scalar) name of HTML file to convert to wikitext
+ html      - (scalar) HTML source to convert
+ base_url  - (scalar) base URL used to make absolute URLs
+ dialect   - (scalar) wiki engine target (default is 'MediaWiki')
+ benchmark - (scalar) true value indicates that benchmarks
+                      should be recorded
+
+Supported wiki dialects are in the HTML::WikiConverter::Dialect
+namespace. The default installation of HTML::WikiConverter includes
+HTML::WikiConverter::Dialect::MediaWiki.
 
 If both the 'file' and 'html' attributes are specified, only
-the 'file' attribute will be used.
+the 'file' attribute will be used. The HTML source (from the 'html'
+or 'file' attribute) is parsed immediately in the constructor.
+
+All attributes (including those not listed above) will be passed
+to the dialect class when creating a tag handler instance.
 
 =cut
 
@@ -213,6 +245,30 @@ Returns the root HTML::Element of the HTML tree
 =cut
 
 sub root { shift->{root} }
+
+=item B<dialect>
+
+=item B<tag_handler>
+
+=item B<tag_handler_class>
+
+  $dialect       = $wc->dialect
+  $handler       = $wc->tag_handler
+  $handler_class = $wc->tag_handler_class
+
+Related methods that return information about the current dialect
+being used to process incoming HTML. The C<dialect> method returns
+whatever value was passed as the C<dialect> property in the C<new>
+constructor.
+
+The C<tag_handler_class> method returns the name of the dialect class,
+such as "HTML::WikiConverter::Dialect::MediaWiki".
+
+The C<tag_handler> returns the instance of the tag handler class that
+is being used for HTML conversion.
+
+=cut
+
 sub dialect { shift->{dialect} }
 sub tag_handler { shift->{tag_handler} }
 sub tag_handler_class { shift->{tag_handler_class} }
@@ -255,7 +311,9 @@ HTML input provided to C<new> because of internal processing done by
 HTML::TreeBuilder, namely that all start tags are closed, HTML, BODY,
 and HEAD tags are automatically wrapped around the provided HTML
 source (if not already present), tags are converted to lowercase,
-attributes are quoted, etc.
+tag attributes are quoted, etc.
+
+This method is useful for debugging.
 
 =cut
 
